#include <GL/glu.h>
#include <GL/glut.h>
#include <GL/glext.h>
#include <stdio.h>
#include <math.h>
#define sqr(a) (a)*(a)

// Initial square position and size
GLfloat x = 40.0f;
GLfloat y = 40.0f;
GLfloat r = 15;
GLfloat yangle = 80.0;
GLfloat xangle = 0.0;
char direction = 'd';
int gem = 1;

// Step size in x and y directions
// (number of pixels to move each time)
GLfloat xstep = 10.0f;
GLfloat ystep = 10.0f;

// Keep track of windows changing width and height
GLfloat width;
GLfloat height;

void RenderPacman(GLfloat x, GLfloat y)
{
	int i, a, b;

	glColor3f(1.0f, 0.0f, 0.0f);
	glPushMatrix();
	glTranslatef(x, y, 0.0);
	glutWireSphere(10, 10, 10);
	glPopMatrix();
/*        fprintf(stderr, "x = %f, y = %f, r = %f\n", x, y, r);*/
}

void RenderBoard()
{
	GLfloat width = 100, height = 100;
	glColor3f(0.0f, 0.0f, 1.0f);
	glRectf(-width, -height, -width+5, height);
	glRectf(-width, -height, width, -height+5);
	glRectf(width-5, -height, width, height);
	glRectf(-width, height-5, width, height);
	glRectf(-2, height, 3, 30);
	glRectf(-2, -height, 3, -30);
	if (gem) {
		glColor3f(1.0, 1.0, 0.0);
		glutWireCube(20.0);
	}
}

void RenderScene(void)
{
        // Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0, -250.0, 50.0, //eye
		  0.0, 1.0, 0.0, // center
		  0.0, 0.0, 1.0); // up


	glRotatef(yangle, 1.0, 0.0, 0.0);
	glRotatef(xangle, 0.0, 1.0, 0.0);

	RenderBoard();
	RenderPacman(x, y);

        // Flush drawing commands and swap
        glutSwapBuffers();
}

int IsWall(GLfloat x, GLfloat y, GLfloat xstep, GLfloat ystep)
{
	GLfloat nx;
	GLfloat width = 100, height = 100;
/*        fprintf(stderr, "%f, %f, %f, %f\n", x, y, xstep, ystep);*/

	if (xstep) {
		if (((y >= 30 && y < height) || (y <= -30 && y > -height)) &&
		    ((xstep < 0 && (x - r) > 3 && (x - r + xstep) < -2) ||
		     (xstep > 0 && (x + r) < -2 && (x + r + xstep) > 3)))
			return 1;
	}

	if (ystep && (x - r) < 3 && (x + r) > -2) {
		if ((ystep > 0 && (y + r + ystep) > 30) ||
		    (ystep < 0 && (y - r + ystep) < -30))
			return 1;
	}
	return 0;
}

/*
 * a - left (104)
 * s - down (106)
 * w - up (107)
 * d - right (108)
 */
void KeyboardFunc(unsigned char key, int _x, int _y)
{
	GLfloat width = 100, height = 100;
	direction = key;
	switch (key) {
	case 'j':
		xangle += 4;
		break;
	case 'l':
		xangle -= 4;
		break;
	case 'i':
		yangle += 4;
		break;
	case 'k':
		yangle -= 4;
		break;
	case 'a':
		if (!IsWall(x, y, -xstep, 0) && x > (-width + r + xstep))
			x -= xstep;
		break;
	case 'd':
		if (!IsWall(x, y, xstep, 0) && x < (width - r - xstep))
			x += xstep;
		break;
	case 's':
		if (!IsWall(x, y, 0, -ystep) && y > (-height + r + ystep))
			y -= ystep;
		break;
	case 'w':
		if (!IsWall(x, y, 0, ystep) && y < (height - r - ystep))
			y += ystep;
		break;
	default:
		direction = 'd';
	}
	if (sqr(x) + sqr(y) < sqr(r))
		gem = 0;
        glutPostRedisplay();
	fprintf(stderr, "[%f, %f, %f, %f]\n", x, y, xangle, yangle);
}

void ChangeSize(int w, int h)
{
        GLfloat aspectRatio;

        // Prevent a divide by zero
        if(h == 0) h = 1;

	width = w;
	height = h;
        // Set Viewport to window dimensions
        glViewport(0, 0, width, height);

        // Reset coordinate system
        glMatrixMode(GL_PROJECTION);
	// Calculate aspect ratio of the window
	gluPerspective(45.0, width / height, 1.0, 400.0);
	// Enables clearing of depth buffer
	glClearDepth(1.0);
	// Set type of depth test
	glDepthFunc(GL_LEQUAL);
	// Enable depth testing
	glEnable(GL_DEPTH_TEST);
	// Enable smooth color shading
	glShadeModel(GL_SMOOTH);

}

int main(int argc, char* argv[]) {
        glutInit(&argc, argv);
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
        glutInitWindowSize(300, 300);
        glutCreateWindow("pacman");
        glutDisplayFunc(RenderScene);
        glutKeyboardFunc(KeyboardFunc);
	glutReshapeFunc(ChangeSize);

        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

        glutMainLoop();

        return 0;
}
